package serverP2P;
import java.util.Vector;
import java.io.File;
import java.io.IOException;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.nio.file.Paths;
import java.nio.file.Files;
import protocolP2P.ProtocolP2PDatagram;
import protocolP2P.RequestResponseCode;
import protocolP2P.Payload;
import protocolP2P.LoadRequest;
import protocolP2P.FileList;
import protocolP2P.FilePart;
import exception.InternalError;
import exception.ProtocolError;
import exception.SizeError;
import exception.TransmissionError;
import exception.VersionError;
import remoteException.EmptyDirectory;
import remoteException.InternalRemoteError;
import remoteException.NotFound;
import remoteException.ProtocolRemoteError;
import remoteException.VersionRemoteError;
import remoteException.EmptyFile;
import java.util.Arrays;


/** Implementation of P2P-JAVA-PROJECT VERSION 1.0 protocol for UDP.
 * @author	Louis Royer
 * @author	Flavien Haas
 * @author	JS Auge
 * @version	1.0
 */
public class ServerManagementUDP implements Runnable {

	private String[] fileList;
	private String baseDirectory;
	private int UDPPort;
	private DatagramSocket socket;

	/** Constructor for UDP implementation, with baseDirectory and UDPPort parameters.
	 * @param baseDirectory	the root directory where files are stored
	 * @param UDPPort	the server will listen on this port
	 */
	public ServerManagementUDP(String baseDirectory, int UDPPort) {
		this.baseDirectory = baseDirectory;
		this.UDPPort = UDPPort;
		initFileList();
		try {
			socket = new DatagramSocket(UDPPort);
		} catch (SocketException e) {
			System.err.println("Error: cannot listen on port " + UDPPort);
			System.exit(-1);
		}
	}

	/** Implementation of runnable. This methods allows to run the server.
	 */
	public void run() {
		while(true) {
			try {
				ProtocolP2PDatagram pd = ProtocolP2PDatagram.receive(socket);
				Payload p = pd.getPayload();
				switch (p.getRequestResponseCode()) {
					case LOAD_REQUEST:
						System.out.println("Received LOAD_REQUEST");
						assert p instanceof LoadRequest : "payload must be an instance of LoadRequest";
						if (!(p instanceof LoadRequest)) {
							sendInternalError(pd);
						} else {
							String filename = ((LoadRequest)p).getFilename();
							try {
								byte[] load = Files.readAllBytes(Paths.get(baseDirectory + filename));
								if (Arrays.binarySearch(fileList, filename) >= 0) {
									try {
										if (load.length == 0) {
											(new ProtocolP2PDatagram(new Payload(RequestResponseCode.EMPTY_FILE))).send(socket, pd);
										} else {
											(new ProtocolP2PDatagram((Payload)(new FilePart(filename, load.length, 0, load)))).send(socket, pd);
										}
									} catch (Exception e2) {
										System.err.println(e2);
									}
								} else {
									throw new IOException(); // to send a NOT_FOUND in the catch block
								}
							} catch (IOException e) {
								try {
									(new ProtocolP2PDatagram(new Payload(RequestResponseCode.NOT_FOUND))).send(socket, pd);
								} catch (Exception e2) {
									System.err.println(e2);
								}
							}
						}
						break;
					case LIST_REQUEST:
						System.out.println("Received LIST_REQUEST");
						try {
							if (fileList.length == 0) {
								System.err.println("Sending EMPTY_DIRECTORY");
								(new ProtocolP2PDatagram(new Payload(RequestResponseCode.EMPTY_DIRECTORY))).send(socket, pd);
							} else {
								System.out.println("Sending LIST_RESPONSE");
								(new ProtocolP2PDatagram((Payload)(new FileList(fileList)))).send(socket, pd);
							}
						} catch (Exception e2) {
							System.err.println(e2);
						}
						break;
					default:
						sendInternalError(pd);
				}
			} catch (NotFound e) {
			} catch (EmptyDirectory e) {
			} catch (InternalRemoteError e) {
			} catch (VersionRemoteError e) {
			} catch (ProtocolRemoteError e) {
			} catch (IOException e) {
			} catch (TransmissionError e) {
			} catch (ProtocolError e) {
			} catch (VersionError e) {
			} catch (InternalError e) {
			} catch (SizeError e) {
			} catch (EmptyFile e) {
			}
		}
	}

	/** Initialize local list of all files allowed to be shared.
	 */
	private void initFileList() {
		File folder = new File(baseDirectory);
		Vector<String> v = new Vector<String>();
		File[] files = folder.listFiles();
		/* Add non-recursively files's names to fileList */
		for (File f : files) {
			if (f.isFile()) {
				v.add(f.getName());
			}
		}
		fileList = new String[v.size()];
		v.toArray(fileList);
	}


	
	/** Send an internal error message.
	 * @param pd	ProtocolP2PDatagram to respond
	 */
	private void sendInternalError(ProtocolP2PDatagram pd) {
		try {
			(new ProtocolP2PDatagram(new Payload(RequestResponseCode.INTERNAL_ERROR))).send(socket, pd);
		} catch (Exception e) {
			System.err.println(e);
		}
	}

}

