package tools;
import exception.SizeError;
/** Helper to manipulate byte[].
 * @author	Louis Royer
 * @author	Flavien Haas
 * @author	JS Auge
 * @version	1.0
 */
public class BytesArrayTools {
	/** Write int in a bytearray
	 * @param array	the array to write
	 * @param start	where to begin writting
	 * @param value	int to write
	 */
	public static void write(byte[] array, int start, int value) {
		for(int i=0;i<4;i++) {
			array[start + i] = (byte) ((value >> (8 * (3 - i))) & 0xFF);
		}
	}
	/** Write int in a bytearray 16 bits version
	 * @param array	the array to write
	 * @param start	where to begin writting
	 * @param value	int to write
	 * @throws SizeError
	 */
	public static void write16Bits(byte[] array, int start, int value) throws SizeError {
		assert (value & 0xFFFF0000) == 0x00000000 : "Wrong size for value, value should be only 16 bits";
		if ((value & 0xFFFF0000) != 0x00000000){
			throw new SizeError();
		}
		for(int i=0;i<2;i++) {
			array[start + i] = (byte) ((value >> (8 * (1 - i))) & 0xFF);
		}
	}

	/** Write long in a bytearray
	 * @param array	the array to write
	 * @param start	where to begin writting
	 * @param value	long to write
	 */
	public static void write(byte[] array, int start, long value) {
		for(int i=0;i<8;i++) {
			array[start + i] = (byte) ((value >> (8 * (7 - i))) & 0xFF);
		}
	}

	/** Read int from a bytearray
	 * @param array	the array to read
	 * @param start	where to begin reading
	 * @return value read as int
	 * @throws SizeError
	 */
	public static int readInt(byte[] array, int start) throws SizeError {
		int size = 0;
		for(int i=0;i<4;i++) {
			size |= ((int)array[start + i]) << (8* (3 -i));
		}
		if (size < 0) {
			// Size in array is probably correct
			// but we cannot store it into a int (or this will be negative)
			throw new SizeError();
		}
		return size;
	}

	/** Read int from a bytearray
	 * @param array	the array to read
	 * @param start	where to begin reading
	 * @return value read as int
	 */
	public static int readInt16Bits(byte[] array, int start) {
		int size = 0;
		for(int i=0;i<2;i++) {
			try {
				size |= ((int)array[start + i]) << (8* (1 -i));
			} catch (ArrayIndexOutOfBoundsException e){
				// 0 padding
			}
		}
		return size;
	}

	/** Read long from a bytearray
	 * @param array	the array to read
	 * @param start	where to begin reading
	 * @return value read as long
	 * @throws SizeError
	 */
	public static long readLong(byte[] array, int start) throws SizeError {
		long size = 0;
		for(int i=0;i<8;i++) {
			size |= ((int)array[start + i]) << (8* (7 - i));
		}
		if (size < 0) {
			// Size in array is probably correct
			// but we cannot store it into a int (or this will be negative)
			throw new SizeError();
		}
		return size;
	}

}
